"""
Sequences for various board actions
These methods are generators that will yield as first element the number
of actions (other yields) excluding it self
They will then perform the next action in row (setting registers or reading values from boards etc)
and will yield True or False depending on the result of the action performed
"""
import logging

from . import *
from .utils import get_board_config
import json
from PyQt4 import QtGui, QtCore
from ...kcgwidget import PopupDialog
import os
import traceback


def read_sequence(board_version):
    #print("sequence ",board_version)
    filename = os.path.join(os.path.dirname(__file__), "sequences", "sequences_" + str(board_version)+".json")
    if not os.path.isfile(filename):
        logging.error("No sequence for Board Version {}".format(board_version))
        print("No sequence for Board Version {}".format(board_version))
        return {}
    try:
        with open(filename) as f:
            data = json.load(f)
            
    except Exception as inst:
        print("Error in Sequences {}".format(board_version))
        traceback.print_exc()
        logging.error("Error in Sequences {}".format(board_version))
        logging.error(type(inst))
        logging.error(inst.args)
        return {}
    return data

def run_sequence(board_id, sequence, progressbar=None):
    NUMBER = len(sequence)
    if progressbar != None:
        progressbar.setRange(0, NUMBER)
    for i, entry in enumerate(sequence):
        
        if entry[2] != "":
            # Dialog
            dialog = PopupDialog("{0} (on board {1})".format(entry[2], board_id))
            dialog.exec_()
            dialog.deleteLater()

            if not dialog.get_return_value():
                logging.info("sequence canceled")
                return False

            logging.info(entry[2])
        
        pci.write(board_id, entry[0], entry[1])        
        if progressbar != None: progressbar.setValue(i) 
        if entry[3] != "": logging.info(entry[3])
        logging.info(entry[1]+":  " + str(pci.read(board_id, 1, entry[1])[0]))

        if len(entry) > 4:
            get_board_config(board_id).update(entry[4], int(entry[5]), write=False)
        if len(entry) > 6:
            get_board_config(board_id).update(entry[6], int(entry[7]), write=False)

        QtGui.qApp.processEvents()
        if get_board_config(board_id).is_KAPTURE2():
            time.sleep(0.1)
        else:
            time.sleep(0.5)
    return True




