from .tcp2 import Socket
import traceback
import time

class DeltaPower():
    def __init__(self, ip, verbose=False):
        self.verbose = verbose
        self.sock = Socket(ip, 8462)
        if not self.sock.connect():
            raise RuntimeError("Not Connected")
            return

        self._send_command("SYSTEM:REMOTE:CV:STATUS REM")
        self._send_command("SYSTEM:REMOTE:CC:STATUS LOC")
        

    def write_Current(self, value):
        self._send_command("SOURCE:CURRENT %s" % value)

    def write_Voltage(self, value):
        self._send_command("SOURCE:VOLTAGE %s" % value)


    def _send_command (self, _cmd_str):
        """
            Send cleartext command using the SCPI protocol and the predefined commandset.
            Adds the `newline` terminator character : ``\\\
        """
        argout = None
        try:
            _t0 = time.time()

            print("In _send_command for cmd %s" % _cmd_str)
            _cmd = _cmd_str.rstrip() + "\n"
            
            self.sock.write(_cmd, timeout=2.0)

            '''
            try:
                _cmd.rindex('?')    # RSVP
            except ValueError: # no answer requested
                self.sock.write(_cmd, timeout=2.0)
                #~ self.info_stream("No answer required")

            else:
                try:
                    argout = self.sock.write_readline(_cmd, timeout=2.0)
                    
                    if self.verbose: print("Answer is %s" % argout)
                except Exception as ex:
                    if self.verbose: 
                        print("Exception in _send_command. 1")
                        traceback.print_exc()
                    pass
            '''
            
            # check the power supply error queues
            _error_list = ''
            """
            _errcmd = 'SYSTem:ERRor?\n'
            _error_msg = self.sock.write_readline(_errcmd, timeout=4.0)
            while _error_msg != "0,None":
                _error_list += "PS error for cmd " + _cmd.rstrip() + " : " + _error_msg + '\n'
                print("PS error for cmd %s: %s" % (_cmd.rstrip(), _error_msg))
                
                _error_msg = self.sock.write_readline(_errcmd)
            """
            _duration = time.time() - _t0
            if self.verbose:
                if _duration > 0.005:
                    print("Cmd execution time : %g ms) " % (_duration * 1000))

                # throw an exception when errors are detected
                if _error_list != "":
                    print("_error_msg (if any): %s" % _error_list)

        # catch communication errors and time-outs
        except Exception as ex:            
            if self.verbose:
                print("Exception in _send_command.")
                traceback.print_exc()

           
        return argout