import gevent
from gevent import socket, event, queue, lock
import time

import traceback

class Socket:

    def __init__(self, host, port,
                 eol='\n',      # end of line for each rx message
                 timeout=5.,    # default timeout for read write
                 ):
        self._host = host
        self._port = port
        self._fd = None
        self._timeout = timeout
        self._connected = False
        self._eol = eol
        self._data = ''
        self._lock = lock.Semaphore()
        
    def closeEvent(self, event):    
        self.close()
        super(Socket, self).closeEvent(event)
        
    def connect(self, host=None, port=None):
        try:
            local_host = host or self._host
            local_port = port or self._port

            if self._connected:
                self._fd.close()


            self._fd = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            self._fd.settimeout(1)
            self._fd.connect((local_host, local_port))
            self._fd.setsockopt(socket.IPPROTO_TCP, socket.TCP_NODELAY, 1)
            self._connected = True
            self._host = local_host
            self._port = local_port
            self._data = ''
            return True
        except:
            self._connected =False
            print('Socket Connect Error Traceback')
            traceback.print_exc()
            return False

    def close(self):
        if self._connected:
            self._fd.shutdown(socket.SHUT_RDWR)
            self._fd.close()
            self._data = ''
  
    def write(self, msg, timeout=None):
        with self._lock:
            self._write(msg)

       

    def flush(self):
        self._data = ''

    def _write(self, msg, timeout=None):
        if not self._connected:
            if not self.connect():
                print('not connected')
                return
        self._fd.sendall(msg.encode())
