from PyQt4 import QtGui
from PyQt4.QtGui import QFont, QTableWidget


from ..base import kcgwidget as kcgw
from ..base.backend import board
from ..base.backend.board import available_boards
from ..base import backendinterface as bif
from ..base.groupedelements import Elements
from ..base.globals import glob as global_objects
from .. import config

import numpy as np

__widget_id__ = None

updateData = None

class Calibration(kcgw.KCGWidgets):
    def __init__(self, parent=None):
        super(Calibration, self).__init__()
        self.par = parent
        self.board_config =  board.get_board_config(available_boards[0])
        self.storage = []
        self.latest = []
        self.mode = 1
        self.layout = QtGui.QGridLayout()
        self.setLayout(self.layout)
        lineindex = 0
        #self.layout.addItem(QtGui.QSpacerItem(10, 15), lineindex, 1)
        #lineindex += 1


        #self.layout.addWidget(self.createLabel("Calibration Helper"), lineindex, 0,1,7)
        hint = "Updates everytime a plot-widget in train view is updated."
        #lineindex += 1
        

        self.modeBaseline = self.createCheckbox(text="Baseline", connect=self.changeMode)
        self.modeBaseline.setChecked(True)
        self.modePeaks = self.createCheckbox(text="Peaks", connect=self.changeMode)
        self.group = QtGui.QButtonGroup()
        self.group.addButton(self.modeBaseline, 1)
        self.group.addButton(self.modePeaks, 2)
        self.group.setExclusive(True)

        self.layout.addWidget(self.createLabel("Modus "), lineindex, 0)
        self.layout.addWidget(self.modeBaseline, lineindex, 1)
        self.layout.addWidget(self.modePeaks, lineindex, 2)
        lineindex += 1
        self.layout.addWidget(self.createLabel(hint), lineindex, 0,1,7)
        lineindex += 1

        self.layout.addWidget(self.createLabel("Last Data"), lineindex, 0)
        lineindex += 1
        self.labelData = self.createLabel("no Data")
        self.labelData.setWordWrap(True)
        font = QFont("Monospace");
        font.setStyleHint(QFont.TypeWriter);
        self.labelData.setFont(font)
        self.layout.addWidget(self.labelData, lineindex, 0,1,7)
        lineindex += 1

        self.comment = self.createInput('0',width=20)  
        self.comment.setMaximumWidth(100)      
        self.layout.addWidget(self.comment, lineindex, 0)
        self.buttonStore = self.createButton('store', connect= self.store)
        self.layout.addWidget(self.buttonStore, lineindex, 1)
        self.layout.addWidget(self.createLabel("data length"), lineindex, 2)
        self.labelCtr = self.createLabel("0")
        self.layout.addWidget(self.labelCtr, lineindex, 3)
        lineindex += 1

        self.buttonStore = self.createButton('clear storage', connect= self.clear)
        self.layout.addWidget(self.buttonStore, lineindex, 0)
        self.buttonSave = self.createButton('save to file', connect= self.save)
        self.layout.addWidget(self.buttonSave, lineindex, 1)
        lineindex += 1
        #table    = QTableWidget()
        #table.resize(self.board_config.get('adc_number')*50, 250)
        #table.setColumnCount(self.board_config.get('adc_number')*2)
        #table.setHorizontalHeaderLabels(['ADC' +str(v) for v in range(self.board_config.get('adc_number'))] )

        
        #updateData(1)


    def updateData(self, data):
        try:
            self.data = data.train(adc=[1,2,3,4])  
            
            self.latest = []    
            disp = self.makeHeader() + '\n'
            if self.mode == 1: #baseline 

                for i in range(self.board_config.get('adc_number')): #enumerate(self.data):
                    mu = np.mean(self.data[i])
                    si = np.std(self.data[i])
                    self.latest.extend([mu, si])
                    disp = disp + ' {:8.3f}, {:6.3f} |'.format(mu, si)
                disp = disp +"\n"
            else:
                
                for i in range(self.board_config.get('adc_number')): #enumerate(self.data):
                    low = np.min(self.data[i][4:-4])
                    high = np.max(self.data[i][4:-4])
                    #print(low, high, (high-low)/2)
                    yl = []
                    yh = []
                    split =  low+(high-low)/2
                    yl = self.data[i][np.where(self.data[i] < split)]
                    yh = self.data[i][np.where(self.data[i] > split)]
                    #for v in self.data[i]:
                    #    if v < split: yl.append(v)
                    #    else:         yh.append(v)
                    #print('1')
                    mul = np.mean(yl)
                    sil = np.std(yl)
                    muh = np.mean(yh)
                    sih = np.std(yh)
                    self.latest.extend([mul, sil, muh, sih])
                    disp = disp + ' {:8.3f}, {:6.3f} ; {:8.3f}, {:6.3f} |'.format(mul, sil, muh, sih)
                disp = disp +"\n"


        except Exception as e:
            disp = str(e)
        self.labelData.setText(disp)

    def changeMode(self):
        self.clear()
        self.mode = self.group.checkedId()

    def store(self):   
        if len(self.latest) == 0:
            self.labelData.setText("no data to store")   
            return
        try:
            com = [float(str(self.comment.text()))]
            com.extend(self.latest)
        
            self.storage.append(com)
            self.labelCtr.setText(str(len(self.storage)))
        except Exception as e:
            self.labelData.setText(str(e))
            return

    def clear(self):
        self.storage = []
        self.labelCtr.setText(str(len(self.storage)))

    def save(self):
        if len(self.storage) == 0:
            self.labelData.setText("no data in storage")
            return
        fileName = QtGui.QFileDialog.getSaveFileName(self, 'Save Calibration Data', config.install_path, selectedFilter='*.csv')
        if fileName:
            print(fileName)
            header =''
            np.savetxt(str(fileName), self.storage, header=self.makeHeader(True), comments='', fmt='%9.3f', delimiter=',')

        pass

    def makeHeader(self, file=False):
        header = " "
        if file:
            header = "         ,"
            if self.mode == 1: 
                for i in range(self.board_config.get('adc_number')):
                    header = header + "        ADC " + str(i) + "    , ,"             
            else:
                for i in range(self.board_config.get('adc_number')):
                    header = header + "                   ADC " + str(i) + "             , ,"
                header = header[:-1] +"\n" +"         ,"
                for i in range(self.board_config.get('adc_number')):
                    header = header + "        lower      ,       upper       ,"
            return header[:-1]

        if self.mode == 1: 
            for i in range(self.board_config.get('adc_number')):
                header = header + "      ADC " + str(i) + "      | "             
        else:
            for i in range(self.board_config.get('adc_number')):
                header = header + "                ADC " + str(i) + "               | "
            header = header +"\n" 
            for i in range(self.board_config.get('adc_number')):
                header = header + "        lower             upper      |"
             
        return header

        

#        d8888 8888888b.   .d8888b.      888       888 d8b      888                   888
#       d88888 888  "Y88b d88P  Y88b     888   o   888 Y8P      888                   888
#      d88P888 888    888 888    888     888  d8b  888          888                   888
#     d88P 888 888    888 888            888 d888b 888 888  .d88888  .d88b.   .d88b.  888888
#    d88P  888 888    888 888            888d88888b888 888 d88" 888 d88P"88b d8P  Y8b 888
#   d88P   888 888    888 888    888     88888P Y88888 888 888  888 888  888 88888888 888
#  d8888888888 888  .d88P Y88b  d88P     8888P   Y8888 888 Y88b 888 Y88b 888 Y8b.     Y88b.
# d88P     888 8888888P"   "Y8888P"      888P     Y888 888  "Y88888  "Y88888  "Y8888   "Y888
#                                                                        888
#                                                                   Y8b d88P
#                                                                    "Y88P"
class AdcWidget(kcgw.KCGWidgets):
    def __init__(self, unique_id, parent):
        super(AdcWidget, self).__init__()

        self.id = unique_id
        self.par = parent

        #self.layout = QtGui.QHBoxLayout()
        self.board_config =  board.get_board_config(available_boards[0])

        self.layout = QtGui.QGridLayout()
        self.outerLayout = QtGui.QVBoxLayout()
        self.outerLayout.addLayout(self.layout)
        self.setLayout(self.outerLayout)

        if not self.board_config.is_KAPTURE2():
            self.layout.addWidget(self.createLabel('Only for KAPTURE 2'))
            return

        lineindex = 0
        adc_number = self.board_config.get('adc_number')

        #self.layout.addWidget(self.createLabel('ADC'), lineindex, 1,1, adc_number+2)
        #lineindex += 1
        for i in range(adc_number):
            self.layout.addWidget(self.createLabel("ADC " + str(i+1)), lineindex, i+1)
        lineindex += 1


        

        # ---- Gain -----
        self.layout.addWidget(self.createLabel('Gain'), lineindex, 0)
        self.gainInput =[] 
        for i in range(adc_number):
            self.gainInput.append(self.createInput("0x{:04x}".format(int(self.board_config.get('adc_gain')[i]))))
            self.layout.addWidget(self.gainInput[i], lineindex, i+1)
        self.board_config.observe(self, self.gainObserver , 'adc_gain')

        self.gainButton = self.createButton("write", connect=self.on_adc_gain_changed)
        self.layout.addWidget(self.gainButton, lineindex, adc_number+1)
        lineindex += 1



        # ---- Offset ----
        self.layout.addWidget(self.createLabel('Offset'), lineindex, 0)
        self.offsetInput = []
        for i in range(self.board_config.get('adc_number')): 
            self.offsetInput.append(self.createInput("0x{:04x}".format(int(self.board_config.get('adc_offset')[i]))))
            self.layout.addWidget(self.offsetInput[i], lineindex, i+1)
        self.board_config.observe(self, self.offsetObserver , 'adc_offset')

        self.offsetButton = self.createButton("write", connect=self.on_adc_offset_changed)
        self.layout.addWidget(self.offsetButton, lineindex, adc_number+1)
        lineindex += 1

        # ---- Buttons ----

        line = QtGui.QFrame()
        line.setFrameStyle(QtGui.QFrame.HLine)
        self.layout.addWidget(line, lineindex, 0, 1, adc_number+2)
        lineindex += 1
        self.layout.addWidget(self.createButton("Calibration Helper", connect=self.toggleCalibration), lineindex, 0)
        lineindex += 1
        
        self.subWindow = Calibration()
        #self.layout.addLayout(self.subWindow.layout, lineindex,0,1,adc_number+2)
        self.outerLayout.addWidget(self.subWindow)
        self.toggleCalibration()
        self.outerLayout.addStretch(1)
        self.setMinimumWidth(self.board_config.get('adc_number')*140+100)
        self.adjustSize()
        #self.layout.addStretch(1)
        self.setWindowTitle("ADC Settings Widget")

        Elements.addItem([
                            "no_board_0", 
                            "acquire_0", 
                            "continuous_read_0"
                         ], 
                         [
                            self.gainButton,
                            self.offsetButton,
                         ]
                         )

    def toggleCalibration(self):
        self.subWindow.setHidden(not self.subWindow.isHidden())
        global updateData
        if self.subWindow.isHidden():
            updateData = None
        else:
            updateData = self.subWindow.updateData
        #self.adjustSize()


    def gainObserver(self, x):
        for i, item in enumerate(self.gainInput):
            item.setText("0x{:04x}".format(int(x[i])))
    def offsetObserver(self, x):
        for i, item in enumerate(self.offsetInput):
            item.setText("0x{:04x}".format(int(x[i])))  

    def closeEvent(self, event):
        global __widget_id__
        __widget_id__ = None
        if not self.board_config.is_KAPTURE2():
            return
            
        self.board_config.unobserve(self, 'adc_gain')
        self.board_config.unobserve(self, 'adc_offset')

        Elements.removeItem(None, [
                                    self.gainButton,
                                    self.offsetButton
                                   ])

        global updateData
        updateData = None

        del self.par.widgets[self.id]


    def on_adc_gain_changed(self):
        try:
            good = True
            out= np.zeros(len(self.gainInput))
            for i, item in enumerate(self.gainInput):
                val = int(str(item.text()),0)
                if val >= 0 and val < 0x7FFF:
                    out[i] = val
                    item.setStyleSheet("color: rgb(0, 0, 0);")
                else: 
                    item.setStyleSheet("color: rgb(255, 0, 0);")
                    good = False


            if good:
                self.board_config.update('adc_gain', out)
        except Exception as e:
            print(e)
            for item in self.gainInput:
                item.setStyleSheet("color: rgb(255, 0, 0);")
            

    def on_adc_offset_changed(self):
        try:
            good = True
            out= np.zeros(len(self.offsetInput))
            for i, item in enumerate(self.offsetInput):
                val = int(str(item.text()),0)
                if val >= 0 and val < 0x1FFF:
                    out[i] = val
                    item.setStyleSheet("color: rgb(0, 0, 0);")
                else: 
                    item.setStyleSheet("color: rgb(255, 0, 0);")
                    good = False


            if good:
                self.board_config.update('adc_offset', out)
        except Exception as e:
            print(e)
            for item in self.offsetInput:
                item.setStyleSheet("color: rgb(255, 0, 0);")


def addAdcWidget():
    global __widget_id__
    if __widget_id__:
        global_objects.get_global('area').widgets[__widget_id__].setFocus()
    else:
        nid = kcgw.idg.genid()
        __widget_id__ = nid
        w = AdcWidget(nid, global_objects.get_global('area'))
        global_objects.get_global('area').newWidget(w, "ADC Widget", nid, widget_type=4, minSize=True)

kcgw.register_widget(QtGui.QIcon(config.icon_path("adc_icon.svg")), "ADC Widget", addAdcWidget, "Ctrl+G")
