"""
This defines the window for initial configuration
"""
from PyQt4 import QtGui, QtCore
from ..base import kcgwidget as kcgw
import os
import sys
if sys.version_info[:3] < (3,0):
    import ConfigParser as configparser
else:
    import configparser
from .. import config
try:
    from pprint import pformat
except ImportError:
    def pformat(string, width=None):
        return string

#  .d8888b.                  .d888d8b        8888888888        888
# d88P  Y88b                d88P" Y8P        888               888
# 888    888                888              888               888
# 888        .d88b. 88888b. 888888888 .d88b. 8888888   88888b. 888888888d888888  888
# 888       d88""88b888 "88b888   888d88P"88b888       888 "88b888   888P"  888  888
# 888    888888  888888  888888   888888  888888       888  888888   888    888  888
# Y88b  d88PY88..88P888  888888   888Y88b 888888       888  888Y88b. 888    Y88b 888
#  "Y8888P"  "Y88P" 888  888888   888 "Y888888888888888888  888 "Y888888     "Y88888
#                                         888                                    888
#                                    Y8b d88P                               Y8b d88P
#                                     "Y88P"                                 "Y88P"

class ConfigEntry(kcgw.KCGWidgets):
    """
    Container for a single entry in the init config window
    """
    def __init__(self, comm='', conf='', type=str, multiline=False, defaultValue=''):
        """
        Initialise an entry
        :param comm: comment
        :param conf: configuration option
        :param type: data type for this option
        :param multiline: if multiline editfield or not
        :return: -
        """
        super(ConfigEntry, self).__init__()
        self.layout = QtGui.QVBoxLayout()
        self.entryLayout = QtGui.QHBoxLayout()
        self.setLayout(self.layout)
        self.toggle = self.createButton("?", connect=self.toggle)
        self.toggle.setFixedWidth(20)

        if len(comm.split("\n")) <= 4:
            self.comment = self.createLabel(comm)
            self.comment.setStyleSheet("color: grey;")
            self.comment.setAlignment(QtCore.Qt.AlignRight)

        else:
            self.commentText = self.createLabel(comm)
            self.commentText.setStyleSheet("color: grey;")
            self.commentText.setAlignment(QtCore.Qt.AlignRight)

            self.commentWidget = QtGui.QWidget()
            self.comment = QtGui.QScrollArea()
            self.comment.setAlignment(QtCore.Qt.AlignRight)
            self.commlayout = QtGui.QVBoxLayout()
            self.commentWidget.setLayout(self.commlayout)
            self.commlayout.addWidget(self.commentText, alignment=QtCore.Qt.AlignRight)
            self.comment.setWidget(self.commentWidget)
            self.comment.text = lambda: self.commentText.text()

        self.config = self.createLabel(conf)
        self.config.setFixedWidth(200)
        self.multiline = multiline
        self.type = type
        if not multiline:
            self.value = self.createInput(width=300)
        else:
            self.value = QtGui.QTextEdit()
            self.value.setFixedWidth(500)
        try:
            self.value.setText(pformat(getattr(config, conf), width=60, indent=4))
        except AttributeError:
            self.value.setText(pformat(config.leval(defaultValue), width=60, indent=4))
        self.layout.addWidget(self.comment)
        self.layout.addLayout(self.entryLayout)
        self.entryLayout.addWidget(self.config)
        self.entryLayout.addWidget(self.value)
        self.entryLayout.addWidget(self.toggle)
        self.comment.hide()

    def toggle(self):
        """
        Toggle the visibility of the comment
        :return:
        """
        self.comment.setHidden(not self.comment.isHidden())

    def set_comment(self, comm):
        """
        Set the comment
        :param comm:
        :return:
        """
        self.comment.setText(comm)

    def set_config(self, conf):
        """
        Set the config option
        :param conf:
        :return:
        """
        self.config.setText(conf)

    def validate(self, mark=False):
        """
        Validate if entered value is valid
        :param mark: mark false entered values?
        :return: -
        """
        if not self.value.isEnabled():
            return True
        if self.multiline:
            try:
                v, e = self.san(str(self.value.toPlainText()))
                if e:
                    self.value.setText(v)
                if type(config.leval(v)) == self.type:
                    self.value.setStyleSheet("background-color: #FFFFFF;")
                    return True
            except (ValueError, config.NoValueException, SyntaxError):
                print('ValueError',v,e)
                pass
        else:
            try:
                v, e = self.san(str(self.value.text()))
                if e:
                    self.value.setText(v)
                if type(config.leval(v)) == self.type:
                    self.value.setStyleSheet("background-color: #FFFFFF;")
                    return True
            except (ValueError, config.NoValueException, SyntaxError):
                print('ValueError',v,e)
                pass
        if mark:
            self.value.setStyleSheet("background-color: #FFAAAA;")
        return False

    def san(self, val):
        """
        Sanitise string entries (adding " or ' and converting mixed ones)
        :param val: the value to sanitise
        :return: sanitised value and errors
        """
        #print(val)
        err = False
        if self.type == str:
            val = val.replace('"', '\'')
            
            if len(val) == 0:
                return "", False
            if val[0] == "'":
                if not val[-1] == "'":
                    val += "'"
                    err = True
            elif val[-1] == "'":
                val = "'" + val
                err = True
            else:
                val = "'" + val + "'"
                err = False
        return val, err

    def __str__(self):
        """
        Convert this entry to a string that can be saved to a configuration file
        :return:
        """
        comment = ''
        for line in str(self.comment.text()).split('\n'):
            comment += "# " + line + '\n'
        if self.value.isEnabled():
            if self.multiline:
                val = self.value.toPlainText()
            else:
                val = self.value.text()
        else:
            val = "'\"disabled\"'"

        val, e = self.san(val)

        return str(comment + self.config.text() + " = " + str(val))


#  .d8888b.                  .d888d8b         .d8888b.         888
# d88P  Y88b                d88P" Y8P        d88P  Y88b        888
# 888    888                888              Y88b.             888
# 888        .d88b. 88888b. 888888888 .d88b.  "Y888b.   .d88b. 888888888  88888888b.
# 888       d88""88b888 "88b888   888d88P"88b    "Y88b.d8P  Y8b888   888  888888 "88b
# 888    888888  888888  888888   888888  888      "88888888888888   888  888888  888
# Y88b  d88PY88..88P888  888888   888Y88b 888Y88b  d88PY8b.    Y88b. Y88b 888888 d88P
#  "Y8888P"  "Y88P" 888  888888   888 "Y88888 "Y8888P"  "Y8888  "Y888 "Y8888888888P"
#                                         888                                888
#                                    Y8b d88P                                888
#                                     "Y88P"                                 888

class ConfigSetup(kcgw.KCGWidgets):
    """
    This is the Config window
    """
    success_signal = QtCore.pyqtSignal()
    machine_comments = ["Bunches per turn of accelerator (integer value)",
                        "Save headerinformation in file (bool value[True/False])",
                        "Revolution time (double value in s eg 3.6825476109875985e-07)"
                        ]
    board_comments = ["Working Channels (List [0,1,2,3,4,5,6,7])"]
    ui_comments = ['NOTE: This value will be overwritten when the language is changed in the gui-settings\n'
                        'possible languages:\n'
                        '"en_GB" - English\n'
                        '"de_DE" - German',
                   '\ndefault_save_location: use "pwd" for current working\n'
                        'directory KCG will always save in a subdirectory to\n'
                        'this given path and save files in this directory',
                   '\ndefault_subdirectory_name_format: this is the\n'
                        'naming scheme for the subdirectory in which the\n'
                        'files are saved. Format of this string:\n'
                        '"{tag1}text{tag2}text" etc.\n'
                        'possible tags:\n'
                        '{dateG} will produce e.g. 04.01.2015\n'
                        '{dateGd} will produce e.g. 04_01_2015\n'
                        '{dateA} will pdoduce e.g. 01-04-2015\n'
                        '{times} will produce e.g. 14_04\n'
                        '{timel} will produce e.g. 14_04_12\n'
                        '{d} the Day in 2 digit format\n'
                        '{m} the Month in 2 digit format\n'
                        '{y} the Year in 4 digit Format\n'
                        '{H} the Hour in 2 digit format\n'
                        '{M} the minute in 2 digit format\n'
                        '{S} the seconds in 2 digit format\n'
                        '{timestamp} unix timestamp without msec\n'
                        '{user} the current logged in user\n'
                        '{sessionname} Ask for session name at startup\n'
                        '{ask} always ask for a foldername\n'
                        'default_subdirectory_name = "{user}_{dateGd}-{timel}_{sessionname}"',
                   '\nreask on cancel in dialog or use {user}_{dateGd}-{timel} as default when cancel is pressed?',
                   '\nShow advanced table view per default? (boolean value)',
                   '\nActivate Epics Widget\n' 
                   'Epics settings need to be in epics.cfg'
                ]
    logging_comments = ['List of Entries that are default to save in Log\n'
                            'Possible Values are:\n'
                            '"Number of Turns" \n'
                            '"Number of Skipped Turns" \n'
                            '"Number of Acquisitions" \n'
                            '"Time between Acquisitions" \n'
                            '"Pilot Bunch Simulator" \n'
                            '"Header saved" \n'
                            '"T/H Delay" \n'
                            '"25ps Delay" \n'
                            '"ADC Delays" \n'
                            '"25ps Delay 2" \n'
                            '"T/H Delay 2" \n'
                            '"Delay Cascade" \n'
                            '"Delay Cascade 25ps" \n'
                            '"Working Channels"\n'
                            '"shiftFMC2"\n'
                            'NOTE: These entries have to match the aforementioned strings exactly'
                    ]
    machine_configs = ['bunches_per_turn',
                       'save_header',
                       'tRev']
    board_configs = ['working_channels']
    ui_configs = ['language',
                  'default_save_location',
                  'default_subdirectory_name',
                  'force_ask',
                  'show_advanced_control',
                  'use_epics']
    logging_configs = ['default_log_entries']
    machine_types = [int, bool, float]
    board_types = [list]
    ui_types = [str, str, str, bool, bool,bool]
    logging_types = [list]

    def __init__(self, restart=False):
        """
        Initialise this config window
        :return:
        """
        super(ConfigSetup, self).__init__()
        self.restart = restart
        self.configs = []
        self.result = False
        self.resize(800, 600)
        self.mainlayout = QtGui.QVBoxLayout()

        defaultConfig = configparser.ConfigParser()
        defaultConfig.optionxform = str
        defaultConfig.read(os.path.join(config.install_path, "default_config.cfg"))
        # self.textEdit = QtGui.QTextEdit()
        # self.layout.addWidget(self.textEdit)
        # with open(os.path.dirname(__file__)+'/../config.cfg', 'r') as f:
        #     self.textEdit.setText(f.read())
        self.setLayout(self.mainlayout)
        self.scrollWidget = QtGui.QScrollArea()
        self.heading = self.createLabel("Initial Configuration Setup")
        self.heading.setStyleSheet("font-size:25pt;")
        self.mainlayout.addWidget(self.heading)
        self.mainlayout.addWidget(self.scrollWidget)
        add = " and restart" if self.restart else ""
        self.saveButton = self.createButton("save"+add, connect=self.save)
        self.mainlayout.addWidget(self.saveButton)

        self.wid = QtGui.QWidget()
        self.layout = QtGui.QVBoxLayout()
        self.wid.setLayout(self.layout)

        self.machine_label = self.createLabel("Machine")
        self.machine_label.setStyleSheet("font-size: 20pt;")
        self.layout.addWidget(self.machine_label)
        self.configs.append("[Machine]")
        for comm, conf, type in zip(self.machine_comments, self.machine_configs, self.machine_types):
            try:
                val = defaultConfig['Machine'][conf]
            except:
                val = ""
            self.configs.append(ConfigEntry(comm, conf, type, defaultValue=val))
            self.layout.addWidget(self.configs[-1])


        self.board_label = self.createLabel("Board")
        self.board_label.setStyleSheet("font-size: 20pt;")
        self.layout.addWidget(self.board_label)
        self.configs.append("\n[Board]")
        for comm, conf, type in zip(self.board_comments, self.board_configs, self.board_types):
            try:
                val = defaultConfig['Board'][conf]
            except:
                val = ""
            self.configs.append(ConfigEntry(comm, conf, type, defaultValue=val))
            self.layout.addWidget(self.configs[-1])


        self.ui_label = self.createLabel("Ui")
        self.ui_label.setStyleSheet("font-size: 20pt;")
        self.layout.addWidget(self.ui_label)
        self.configs.append("\n[Ui]")
        for comm, conf, type in zip(self.ui_comments, self.ui_configs, self.ui_types):
            try:
                val = defaultConfig['Ui'][conf]
            except:
                val = ""
            self.configs.append(ConfigEntry(comm, conf, type, defaultValue=val))
            self.layout.addWidget(self.configs[-1])

        self.logging_label = self.createLabel("Logging")
        self.logging_label.setStyleSheet("font-size:20pt;")
        self.layout.addWidget(self.logging_label)
        
        self.configs.append("\n[Logging]")
        self.logging_entries = []
        for comm, conf, type in zip(self.logging_comments, self.logging_configs, self.logging_types):
            try:
                val = defaultConfig['Logging'][conf]
            except:
                val = ""
            if conf == 'default_log_entries':
                self.configs.append(ConfigEntry(comm, conf, type, multiline=True, defaultValue=val))
            else:
                self.configs.append(ConfigEntry(comm, conf, type, defaultValue=val))
            self.layout.addWidget(self.configs[-1])
            self.logging_entries.append(self.configs[-1])

        self.scrollWidget.setWidget(self.wid)

    def save(self):
        """
        Save the configuration entered in the config window to a configuration file
        namely: os.path.expanduser("~")+"/.kcg/config.cfg"
        This also makes shure the entered information is valid to some level
        :return:
        """
        if not os.path.isdir(config.config_path()):
            os.mkdir(config.config_path())

        abort = False
        for item in self.configs:
            if type(item) == str:
                continue
            if not item.validate(mark=True):
                abort = True
        if abort:
            return

        with open(config.config_path('config.cfg'), 'w+') as f:
            for item in self.configs:
                f.write(str(item)+'\n')
        self.result = True
        self.success_signal.emit()
        self.close()


