from PyQt4 import QtGui, QtCore
from PyQt4.QtGui import QFont, QTableWidget


from ..base import kcgwidget as kcgw
from ..base.backend import board
from ..base.backend.board import available_boards
from ..base import backendinterface as bif
from ..base.groupedelements import Elements, cuda_windows
from ..base.globals import glob as global_objects
from .. import config

import numpy as np
import subprocess
import os


__widget_id__ = None




class CudaWidget(kcgw.KCGWidgets):
    def __init__(self, unique_id, parent):
        super(CudaWidget, self).__init__()

        self.id = unique_id
        self.par = parent

        self.process = QtCore.QProcess(self)
        self.process.readyRead.connect(self._processOut)
        self.process.readyReadStandardOutput.connect(self._processOut)
        self.process.finished.connect(self._processFinished)
        self.process.error.connect(self._processError)

        cuda_windows = self.updateData

        #self.layout = QtGui.QHBoxLayout()
        self.board_config =  board.get_board_config(available_boards[0])

        self.layout = QtGui.QGridLayout()
        self.outerLayout = QtGui.QVBoxLayout()
        self.outerLayout.addLayout(self.layout)
        self.setLayout(self.outerLayout)

        
        lineindex = 0
        self.layout.addWidget(self.createLabel("Ausgabe Verzeichnis"), lineindex, 0)
        self.outDir = self.createInput(config.working_path('processed'))
        self.layout.addWidget(self.outDir, lineindex, 1,1,2)
        lineindex += 1

        self.layout.addWidget(self.createLabel("Plot Verzeichnis"), lineindex, 0)
        self.plotDir = self.createInput(config.working_path('plots'))
        self.layout.addWidget(self.plotDir, lineindex, 1,1,2)
        lineindex += 1

        self.layout.addWidget(self.createLabel("Bunch"), lineindex, 0)
        self.bunchnr = self.createSpinbox(-1,184)
        self.layout.addWidget(self.bunchnr, lineindex, 1,1,2)
        lineindex += 1

        self.layout.addWidget(self.createLabel("Calibration file"), lineindex, 0)
        self.calFile = self.createInput(config.config_path('calibration.npy'))
        self.layout.addWidget(self.calFile, lineindex, 1,1,2)
        lineindex += 1

        self.output = QtGui.QTextEdit()
        self.layout.addWidget(self.output, lineindex, 0,1,3)
        lineindex += 1

        self.layout.addWidget(self.createLabel("Data file"), lineindex, 0)
        self.file = self.createInput(config.working_path('1528363637.997.out'))
        self.layout.addWidget(self.file, lineindex, 1,1,2)
        lineindex += 1

        self.layout.addWidget(self.createLabel("additional parameters"), lineindex, 0)
        self.params = self.createInput("-hdf")
        self.layout.addWidget(self.params, lineindex, 1,1,2)
        lineindex += 1

        self.checkAuto = self.createCheckbox('auto run new data')
        self.layout.addWidget(self.checkAuto, lineindex, 0)
        self.layout.addWidget(self.createButton('manuel', connect=lambda: self.run(self.file.text())), lineindex, 1)
        lineindex += 1
       
        self.outerLayout.addStretch(1)
        self.adjustSize()
        #self.layout.addStretch(1)
        self.setWindowTitle("CUDA Peak-Reconstruction Widget")


    def updateData(self, filename):
        self.file.setText(filename)
        if self.checkAuto.isChecked():
            self.run(filename)
        

    def run(self, filename):
        if self.process.state() == QtCore.QProcess.NotRunning:
            self.process.setProcessChannelMode(QtCore.QProcess.MergedChannels)
            param = ["-u", os.path.join(config.install_path,"extensions","PeakReconstruction", "PeakReconstuction.py"), "-f", filename, "-o", self.outDir.text(), "-p", self.plotDir.text(), "-b", str(self.bunchnr.value())]
            if self.params.text() != "":
                param.extend(self.params.text().split(' '))
            self.process.start("python3", param)
            # -u is for python to force it to unbuffered print
            
            self.process.waitForStarted()
            
        else:
            cursor = self.output.textCursor()
            cursor.movePosition(cursor.End)
            cursor.insertText("process still running! {} will not be processed\n".format(filename))
            self.output.ensureCursorVisible()


    def _processOut(self):
        #print('out')
        cursor = self.output.textCursor()
        cursor.movePosition(cursor.End)
        cursor.insertText(self.process.readAll().data().decode('utf8'))
        self.output.ensureCursorVisible()

    def _processFinished(self):
        pass

    def _processError(self):
        pass

    def closeEvent(self, event):
        global __widget_id__
        __widget_id__ = None
        cuda_windows = None

        del self.par.widgets[self.id]


    

def addCudaWidget():
    global __widget_id__
    if __widget_id__:
        global_objects.get_global('area').widgets[__widget_id__].setFocus()
    else:
        nid = kcgw.idg.genid()
        __widget_id__ = nid
        w = CudaWidget(nid, global_objects.get_global('area'))
        global_objects.get_global('area').newWidget(w, "CUDA Widget", nid, widget_type=4, minSize=True)

kcgw.register_widget(QtGui.QIcon(config.icon_path("cuda_icon.png")), "CUDA Widget", addCudaWidget, "Ctrl+G")
