from PyQt4 import QtGui


from ..base import kcgwidget as kcgw
from ..base.backend import board
from ..base.globals import glob as global_objects
from ..base.backend.board import available_boards

from ..base import backendinterface as bif
from .. import config

__widget_id__ = None


class TestingWidget(kcgw.KCGWidgets):
    def __init__(self, unique_id, parent):
        super(TestingWidget, self).__init__()

        self.id = unique_id
        self.par = parent
        self.board_id = available_boards[0]
        self.board_config = board.get_board_config(self.board_id)

        self.layout = QtGui.QGridLayout()
        self.outerLayout = QtGui.QVBoxLayout()
        self.outerLayout.addLayout(self.layout)
        self.outerLayout.addStretch(1)
        self.setLayout(self.outerLayout)

        self.label1  = self.createInput(str(bif.bk_get_config(self.board_id, 'bunches_per_turn')))
        self.button1 = self.createButton("Write Harmonic Number", connect=lambda x: bif.bk_update_config(self.board_id, "bunches_per_turn", self.label1.text().toInt()[0]))        
        self.board_config.observe(self.label1, lambda x: self.label1.setText(str(x)), 'bunches_per_turn')

        self.label2 = self.createLabel("reg")
        self.label3 = self.createLabel("value")

        self.reg = self.createInput("0x9060")
        self.val = self.createInput("0x00000000")
        self.button2 = self.createButton("write", connect=lambda x: board.pci.write(self.board_id, self.val.text(), self.reg.text()))

        self.button3 = self.createButton("read", connect=self.read)
        self.reg2 = self.createInput("0x9060")
        self.out = self.createLabel("-")

        def update_delay(which, spinbox):
            self.board_config.update(which, getattr(self, spinbox).value())

        self.divLabel = self.createLabel("Clock Div")
        self.divInputTh   = self.createSpinbox(0, 64, start_value=self.board_config.get('clk_div_th'), connect=lambda: update_delay("clk_div_th", "divInputTh"))
        self.divInputAdc  = self.createSpinbox(0, 64, start_value=self.board_config.get('clk_div_adc'), connect=lambda: update_delay("clk_div_adc", "divInputAdc"))
        self.divInputFpga = self.createSpinbox(0, 64, start_value=self.board_config.get('clk_div_fpga'), connect=lambda: update_delay("clk_div_fpga", "divInputFpga"))
        self.divInputCasc = self.createSpinbox(0, 64, start_value=self.board_config.get('clk_div_cascade'), connect=lambda: update_delay("clk_div_cascade", "divInputCasc"))

        self.set_default_button  = self.createButton(text="Write all configs", connect=lambda x: self.board_config.notify_all_observers(write=True))
        self.get_settings_button = self.createButton(text="Read all configs",  connect=lambda x: self.board_config.read_from_board())

        self.convertLabel = self.createLabel("Timescan Point Converter")
        self.convertLabelOut = self.createLabel("")
        self.convertInput    = self.createSpinbox(0, 4000, connect=self.convert)
        #self.button4 = self.createButton("read", )


        self.layout.addWidget(self.button1, 0,0)
        self.layout.addWidget(self.label1, 0,1)

        self.layout.addWidget(self.label2,1,1)
        self.layout.addWidget(self.label3,1,2)
        self.layout.addWidget(self.button2,2,0)
        self.layout.addWidget(self.reg,2,1)
        self.layout.addWidget(self.val,2,2)

        self.layout.addWidget(self.button3,3,0)
        self.layout.addWidget(self.reg2,3,1)
        self.layout.addWidget(self.out,3,2)

        self.layout.addWidget(self.divLabel,4,0)
        self.layout.addWidget(self.createLabel("T/H"),5,0)
        self.layout.addWidget(self.createLabel("ADC"),5,1)
        self.layout.addWidget(self.createLabel("FPGA"),5,2)
        self.layout.addWidget(self.createLabel("Cascade"),5,3)
        self.layout.addWidget(self.divInputTh, 6, 0)
        self.layout.addWidget(self.divInputAdc, 6, 1)
        self.layout.addWidget(self.divInputFpga, 6, 2)
        self.layout.addWidget(self.divInputCasc, 6, 3)

        self.layout.addWidget(self.set_default_button, 7,0)
        self.layout.addWidget(self.get_settings_button, 7,1)

        self.layout.addWidget(self.convertLabel, 8,0,1,2)
        self.layout.addWidget(self.convertInput, 9,0)
        self.layout.addWidget(self.convertLabelOut, 9,1,1,2)

        self.setWindowTitle("Testing Widget")
        #self.setMaximumHeight(100)
        #self.resize(150,300)

    def read(self):
        erg = board.pci.read(self.board_id, reg=self.reg2.text())[0]
        self.out.setText(str(erg))

    def closeEvent(self, event):
        board.get_board_config(self.board_id).unobserve(self.label1, 'bunches_per_turn')
        global __widget_id__
        __widget_id__ = None
        del self.par.widgets[self.id]

    def convert(self):
        val = self.convertInput.value()
        c1 = 15 - int(val/64/6)
        c2 = 23 - (val//64 - int(val/64/6)*6)*4
        f  = val - (val//64)*64
        self.convertLabelOut.setText(" {} | {} | {}".format(c1, c2, f))


def addTestingWidget():
    global __widget_id__
    if __widget_id__:
        global_objects.get_global('area').widgets[__widget_id__].setFocus()
    else:
        nid = kcgw.idg.genid()
        __widget_id__ = nid
        w = TestingWidget(nid, global_objects.get_global('area'))
        global_objects.get_global('area').newWidget(w, "Example Widget", nid, widget_type=4, minSize=True)

kcgw.register_widget(QtGui.QIcon(config.icon_path("einstellungen.svg")), "Testing Widget", addTestingWidget, "Ctrl+r")
