# -*- coding: utf-8 -*-
"""
This Module is the SingleRead window.
"""
from PyQt4 import QtGui

from ..base import kcgwidget as kcgw
from ..base import backendinterface as bif
from ..base.storage import storage
from ..base.globals import glob as global_objects
from ..base.backend.CalibrationUpdater import CalibrationUpdater
from ..base.backend.TimeScan import TimeScan
from .. import config
import time
import os
import traceback
import numpy as np


tr = kcgw.tr

__widget_id__ = None


class UpdateCalibrationWidget(kcgw.KCGWidgets):
    """
    
    """
    def __init__(self, unique_id, parent=None):
        """
        :param unique_id: the id for this widget
        :param parent: parent object
        :return: -
        """
        super(UpdateCalibrationWidget, self).__init__()

        self.id = unique_id
        self.par = parent

        self.updater = None

        self.layout = QtGui.QGridLayout()
        self.outerLayout = QtGui.QVBoxLayout()

        self.setLayout(self.outerLayout)

        # ---------[ Create Labels and corresponding Fields ]---------
        try:
            self.path = bif._bif_update_working_dir()
            #self.path = str(os.path.join(storage.storage.save_location, storage.storage.subdirname, 'TimeScan'))
            self.path = os.path.join(self.path, 'TimeScan')
            print(self.path)
            filelist = np.sort(os.listdir(self.path))
        except:
            print('no TimeScans')
            traceback.print_exc()
            filelist = []
        
        


        self.timescanBox = QtGui.QComboBox(self)
        if len(filelist):
            lineindex = 0
            self.layout.addWidget(self.createLabel("Select TimeScan for CalibrationUpdater"), lineindex, 0)
            lineindex += 1

            for file in filelist:
                if '.scan' in file:
                    self.timescanBox.addItem(file) 
            self.layout.addWidget(self.timescanBox, lineindex, 0)
            

            self.button_run = self.createButton('Update', connect=self.run)
            self.layout.addWidget(self.button_run, lineindex, 1)
        else:
            lineindex = 0
            self.layout.addWidget(self.createLabel("No Timescan found!"), lineindex, 0)
            lineindex += 1



        self.outerLayout.addLayout(self.layout)


    def run(self):
        """
        :return:
        """
        if self.updater is None:
            #file = self.timescanBox[self.timescanBox.currentIndex()]
            file = str(self.timescanBox.currentText())
            self.timescan = TimeScan(os.path.join(self.path, file))
            self.updater = CalibrationUpdater(self.timescan, parent=self, workingChannels=config.working_channels)

    def openWith(self, timescan, bunch=0):
        if self.updater is None:
            self.timescan = timescan 
            self.updater = CalibrationUpdater(self.timescan, parent=self, bunch=bunch, workingChannels=config.working_channels)
            print('opened')

   
    def closeEvent(self, event):
        """
        Event handler to handle the event of closing this window and gracefully delete ids and such
        :param event: the event to process (gets passed by PyQt)
        """
        print('close')
        if self.updater is not None:
            self.updater.on_cancel(0)
        global __widget_id__
        __widget_id__ = None
        del self.par.widgets[self.id]
        

        super(UpdateCalibrationWidget, self).closeEvent(event)


def addUpdateCalibrationWidget():
    """
    Add this widget to the gui.
    This function will actually open the subwindow.
    :return: -
    """
    global __widget_id__
    if __widget_id__:
        global_objects.get_global('area').widgets[__widget_id__].setFocus()
    else:
        nid = kcgw.idg.genid()
        __widget_id__ = nid
        w = UpdateCalibrationWidget(nid, global_objects.get_global('area'))
        global_objects.get_global('area').newWidget(w, "CalibrationUpdater", nid, widget_type=4, minSize=True)  # TODO: proper type
    return global_objects.get_global('area').widgets[__widget_id__]

kcgw.register_widget(QtGui.QIcon(config.icon_path('calib.svg')),
                     "Update Calibration",
                     addUpdateCalibrationWidget,
                     "Ctrl+U"
                     )
